function [ model, misc ] = fit_1d1g1sr_fast( in_data, par, w )

% function [ model, misc ] = fit_1d1g1sr_fast( data, par, w )
%
% Fit a 1d1g1sr model to some 2-D data.
% For details about the model, see compute_post_1d1g1sr.
%
% This is a wrapper of fit_1d1g1sr that:
% ( 1 ) Reduces the data to par.nsamples "well-chosen samples".
% ( 2 ) Calls fit_1d1g1sr on the reduced data
% ( 3 ) (if nargout >1) evaluates misc on the full data
%
% INPUT (data and par are mandatory)
%
% data: a 2-D matrix of >= 0 values
%
% par:  a structure where each field defines a parameter
%   MANDATORY PARAMETER: par.nsamples
%
% w: a 2-D matrix of >=0 values, same size as data
% -> optional weighting, default = equal weights
%
% OUTPUT
%
% model is a structure with fields  
% { 'b0', 'w0d', 'gamma0', 'beta0', 'sigma1', 'V1' }.
%
% ( Implicitly:  b1 = 1 - b0  and  w0g = 1 - w0d. )
%
% misc is a structure with fields
% { 'log_post', 'alldata_log_lkld', 'log_plkld', 'log_pjoint' }
% that are returned by a call to compute_post_1d1g1sr( model, data ).
%
%
% SEE ALSO compute_post_1d1g1sr, fit_1d1g1sr.
  
  
  if nargin < 2
    error( [ mfilename ' needs 2 input arguments.' ] );
  end
  
  if nargin < 3
    w = ones( size( in_data ) );
  end
  
  % This is not supposed to be changed
  GAMMA0_FLOOR = 1 + 1e-10;
  
  % data: save some memory
  global data  nz_ind log_data_nz_ind   negloglike_w0  negloglike_w1;
  
  data = in_data; clear in_data;
  
  % data: make sure it is a 2-D matrix

  size_data = size( data );
  
  if max( find( size_data > 1 ) ) > 2
    error( [ mfilename ': "data" must be a 2-D matrix.' ] );
  end
  
  % Check all data is >= 0
  
  if any( data(:) < 0 )
    error( [ mfilename ': all data must be >=0.' ] );
  end
  
  z_ind  = find( data == 0 );
  z_ind  = z_ind(:);
  
  nz_ind = find( data > 0 );
  nz_ind = nz_ind(:);
  
  log_data_nz_ind = log( data( nz_ind ) );
  log_data_nz_ind = log_data_nz_ind(:);
  
  % Check parameters, fill in default values
  
  check_param( { 'nsamples' }, fieldnames( par ) );
  
  par_default = [];
  
  par_default.model_init = [];
  
  par_default.min_iter = 0;
  par_default.max_iter = +Inf;

  par_default.fminsearch_options = [];
  
  % To make sure the weight of Dirac never falls to zero
  % This is necessary if all training data is > 0,
  % but some testing data is = 0.
  par_default.w0d_floor = 1e-5;
  
  par_default.verbose = 1;
  par_default.debug   = 0;
  
  par = fill_default( par, par_default );
  
  if par.verbose
    disp( ' ' );
    disp( [ mfilename ' parameters:' ] );
    disp( par );
  end

  % Make sure all weights are >=0
  if any( w(:) < 0 )
    error( [ mfilename ' all weights w must be >= 0.' ] );
  end
  
  sum_w = sum( w );
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Wrapper:
  % ( 1 ) Reduce the data to par.nsamples well-chosen samples
  % ( 2 ) Call fit_1d1g1sr
  
  % ( 1 ) Reduce the data to par.nsamples well-chosen samples
  
  reduced_data = repmat( NaN, par.nsamples, 1 );
  
  if all( data == 0 )
    
    reduced_data = 0;
  
  else
    
    % ( 1.1 ) zeroes
    
    pz = sum( w(:) .* ( data(:) == 0 ) ) / sum_w;
    
    if pz == 0
      
      % Case where data does not contain any zero
      
      nz = 0;
      
    else
      
      % In the reduced data, make sure there is at least one zero
      
      nz = max( 1, round( par.nsamples * pz ) );      
      reduced_data( 1:nz ) = 0;
      
    end
    
    % ( 1.2 ) >0 data
    
    % Pick ( par.nsamples - nz ) well-chosen samples
    % i.e. AT REGULAR STEPS ALONG THE CDF OF THE SORTED DATA
    
    ind_nz_data = find( data > 0 );
    
    [ sort_nz_data, ind ] = sort( data( ind_nz_data ) );
    sort_nz_w = w( ind_nz_data( ind ) );
    
    a_cdf = cumsum( sort_nz_w );
    a_cdf = a_cdf / a_cdf( end );
    
    % Build the CDF
    [ a_cdf, ind ] = unique( a_cdf );
    sort_nz_data   = sort_nz_data( ind );
    
    % Inverse it
    n = par.nsamples - nz;
    reduced_data( nz+1:end ) = interp1( a_cdf, sort_nz_data, ( 0.5:1:n ) / n, 'linear',  0 );
        
  end
  
  
  % Sanity check
  
  if any( isnan( reduced_data ) )
    error( [ mfilename ' is NaN-insane.' ] );
  end
  
  if any( isinf( reduced_data ) )
    error( [ mfilename ' is NaN-insane.' ] );
  end
  
  % ( 2 ) Call fit_1d1g1sr
  
  a_par = rmfield( par, 'nsamples' );
  
  model = fit_1d1g1sr( reduced_data, a_par );
    
  % ( 3 ) Create the "misc" structure and return
  
  if nargout > 1
    [ misc.log_post, misc.alldata_log_plkld, misc.log_plkld, misc.log_pjoint ] = compute_post_1d1g1sr( model, data );
  end
  