function   [ fulldata_model, fulldata_misc, summary_model, summary_misc ] = fit_2d2g1sr( in_data, par )

% function [ fulldata_model, fulldata_misc, summary_model, summary_misc ] = fit_2d2g1sr( data, par )
%
% Fit a 2d2g1sr model to some 2-D data.
% For details about the model, see compute_post_2d2g1sr.
%
%
% INPUT (data only is mandatory)
%
% data: a 2-D matrix of >= 0 values
%
% par:  a structure where each field defines a parameter
%
%
% OUTPUT
%
% model is a structure with fields  
% { 'a0', 'b0', 'w00d', 'gamma00', 'beta00', 'w01d', 'gamma01', 'beta01', 'sigma11', 'V11' }.
%
% ( Implicitly:  a1 = 1 - a0, b1 = 1 - b0, w00g = 1 - w00d and w01g = 1 - w01d. )
%
% misc is a structure with fields { 'alldata_log_plkld', 'iter' }
%
%
% SEE ALSO compute_post_2d2g1sr, fit_2d2g1sr_fast

  
  DEBUG_HIST_NBINS = 100;
  
  if nargin < 1
    error( [ mfilename ' needs at least one input argument.' ] );
  end

  if ~exist( 'par', 'var' )
    par = [];
  end
  
  % This is not supposed to be changed
  GAMMA0_FLOOR = 1 + 1e-10;
  
  % data: save some memory
  global data log_data_nz_ind summary fulldata   negloglike_w0  negloglike_w1;
  
  data = in_data; clear in_data;
  
  % data: make sure it is a 2-D matrix
  
  size_data = size( data );
  
  if max( find( size_data > 1 ) ) > 2
    error( [ mfilename ': "data" must be a 2-D matrix.' ] );
  end
  
  % Check all data is >= 0
  
  if any( data(:) < 0 )
    error( [ mfilename ': all data must be >=0.' ] );
  end
  
  fulldata.z_ind  = find( data == 0 );
  fulldata.z_ind  = fulldata.z_ind(:);
  
  fulldata.nz_ind = find( data > 0 );
  fulldata.nz_ind = fulldata.nz_ind(:);
  
  log_data_nz_ind = log( data( fulldata.nz_ind ) );
  log_data_nz_ind = log_data_nz_ind(:);
  
  % Check parameters, fill in default values
  
  par_default = [];
  
  % model_init:
  % - if empty, automatic initialization is used.
  % - if non-empty, this field contains a user-defined 2d2g1sr model to initialize with.
  par_default.model_init = [];
  
  % Alternative: automatic initialization
  par_default.auto_init_n = 30;
  
  % How many frames to retain in the summarized version of the data
  par_default.summary_nframes = min( 1000, size_data( 2 ) );
  
  
  par_default.summary_min_iter = 0;
  par_default.summary_max_iter = +Inf;
  
  par_default.summary_inertia  = 0.5;
  
  % Convergence detected when alldata_log_plkld varied by less than this
  par_default.summary_lkld_conv = 1e-10;
  
  par_default.fulldata_min_iter = 0;
  par_default.fulldata_max_iter = +Inf;
  
  par_default.fulldata_inertia  = 0.5;
  
  % Convergence detected when alldata_log_plkld varied by less than this
  par_default.fulldata_lkld_conv = 1e-10;
  
  par_default.fminsearch_options = [];
  
  % To make sure the weight of Dirac never falls to zero
  % This is necessary if all of the training data is > 0,
  % but some testing data is = 0.
  par_default.wd_floor = 1e-5;
  
  par_default.verbose = 1;
  par_default.debug   = 0;
  
  par = fill_default( par, par_default );
  
  if par.verbose
    disp( ' ' );
    disp( [ mfilename ' parameters:' ] );
    disp( par );
  end
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % ( 1 ) Initialization
  
  
  % Create a summarized version of the data
  
  [ tmp, ind ] = sort( max( data, [], 1 ) );
  
  summary.data = data( :, ind( round( ( 0.5:par.summary_nframes ) / par.summary_nframes * size_data( 2 ) ) ) );
  summary.size_data = size( summary.data );
  summary.nz_ind = find( summary.data(:)  > 0 );
  summary.z_ind  = find( summary.data(:) == 0 );
  summary.log_data_nz_ind = log( summary.data( summary.nz_ind ) );
  
  % Sanity check
  
  if any( summary.size_data ~= [ size_data( 1 )   par.summary_nframes ] )
    error( [ mfilename ': insanity detected while creating the summarized data.' ] );
  end
  
  % Initialize the model
  
  if ~isempty( par.model_init )

    % User-defined init

    if par.verbose
      disp( [ mfilename ': user-defined initialization.' ] );
    end
    
    a_new.model = par.model_init;
    
    
  else

    if par.verbose
      disp( [ mfilename ': automatic initialization.' ] );
	    
    end
    
    % Default, automatic init
    
    a_new.model.a0   = 0.5;  % Equal priors
    a_new.model.b0   = 0.5;  % Equal priors
    a_new.model.w00d = 0.5;  % Equal weights
    a_new.model.w01d = 0.5;  % Equal weights
    
    % Initialize the distributions
    
    
    sort_data = sort( summary.data(summary.nz_ind) );
    
    if par.auto_init_n < 2 
      
      % Single initialization
      thr_list = sort_data( max( 2, round( a_new.model.b0 * numel( sort_data ) ) ) );
      
    else
      
      n1 = max( 1, round( par.auto_init_n / 2 ) );
      
      n2 = max( 0, par.auto_init_n - n1 );
      
      % Multiple initializations
      % (we'll pick the ML one)
      % -> this way we avoid gross mistakes
      %    that would lead to bad local optima
      
      a_max = sort_data( end );
      a_min = sort_data( 1 );
      
      thr_list = a_min + (0.5:n1) / n1 * ( a_max - a_min );
      
      % Add the same number of thresholds,
      % regularly spaced along percentiles
      
      n = numel( sort_data );
      ind = 1 + round( ( 0.5:n2 ) / n2 * ( n - 1 ) );
      
      tmp = sort_data( ind );
      
      thr_list = [ thr_list(:).' tmp(:).' ];
      
    end
      
    thr_list = sort( thr_list );
    
    for a = 1:numel( thr_list )
    
      thr = thr_list( a );
      
      % Gamma pdfs: moment method, same parameters for both
      
      ind = find( sort_data <= thr );
      a_mean = mean( sort_data( ind ) );
      a_var  = var( sort_data( ind ) );
      
      if a_var == 0
	
	log_plkld_list( a ) = -Inf;

      else
	a_new.model.gamma00 = a_mean ^ 2 / a_var;
	a_new.model.beta00  = a_mean / a_new.model.gamma00;
	
	% Ensure that GammaPdf( 0, gamma00, beta00 ) = 0
	
	t00 = a_new.model.gamma00 * a_new.model.beta00;
	
	a_new.model.gamma00 = max( GAMMA0_FLOOR, a_new.model.gamma00 );
	a_new.model.beta00  = t00 / a_new.model.gamma00;
	
	% Same parameters for ( gamma01, beta01 )
	
	a_new.model.gamma01 = a_new.model.gamma00;
	a_new.model.beta01  = a_new.model.beta00;
	
	% Rice pdf: moment method as well
	
	t11 = a_new.model.gamma01 * a_new.model.beta01;
	grt_thr = max( t11, thr );
	grt_ind = find( summary.data > grt_thr );
	if isempty( grt_ind )
	  error( [ mfilename ' could not find data to initialize the shifted Rice!' ] );
	end
	[ a_new.model.sigma11, a_new.model.V11 ] = rice_optimize_sigma_V( summary.data( grt_ind ) - t11 );
	
	% We are almost sure to get those crappy warnings
	% so let's shut if off during initilization
	old_warning_state = warning;
	warning off;
	
	% Update priors
	
	tmp_max = max( summary.data, [], 1 );
	
	a_new.model.a0 = min( 0.9, max( 1e-1, mean( tmp_max <= grt_thr ) ) );
	
	tmp = summary.data( :, tmp_max > grt_thr );
	a_new.model.b0 = min( 0.9, max( 1e-1, mean( tmp(:) <= grt_thr ) ) ); 
	
	% Update mixture weights
	
	tmp = summary.data( :, tmp_max <= grt_thr );
	a_new.model.w00d = min( 0.9, max( par.wd_floor, mean( tmp(:) == 0 ) ) );
	
	tmp = summary.data( :, tmp_max > grt_thr );
	a_new.model.w01d = min( 0.9, max( par.wd_floor, mean( tmp(:) == 0 ) ) );
	
	% Compute lkld
	
	try	
	  [ tmp, tmp, log_plkld_list( a ) ] = compute_post_2d2g1sr( a_new.model, summary.data );
	catch
	  % compute_post_2d2g1sr failed, mostly because of saturation of the besseli(0,x) function in the Rice
	  log_plkld_list( a ) = -Inf;
	end
	
	warning( old_warning_state );
      
	model_list( a ) = a_new;

      end

    end
    
       
    % Pick the ML
    
    [ tmp, max_ind ] = max( log_plkld_list );
    
    a_new = model_list( max_ind );
    
    if par.debug
      figure; plot(thr_list,log_plkld_list,'.-')
      xlabel( 'init threshold' ); ylabel( 'alldata log plkld' );
      hold on; plot( thr_list( max_ind ), log_plkld_list( max_ind ), 'ro' );
      title( strrep( [ mfilename ' initialization' ], '_', '\_' ) );
      
    end
    
  end  % of "if automatic init"
  
  % Check model, if any given
  % Make sure all parameters are defined
  if ~isempty( par.model_init )
     a_new.model = check_2d2g1sr_model( par.model_init );
  end
  
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % ( 2 ) EM training
  % First on the summarized version of the data
  % Second on the full data
  
  summary.best = [];
  fulldata.best    = [];

  summary.iter = 0;
  summary.finished = ( summary.iter >= par.summary_max_iter );
  summary.previous_alldata_log_plkld = [];
  summary.previous_00_X0 = [];
  summary.previous_01_X0 = [];
  
  fulldata.iter = -1;
  fulldata.finished = ( fulldata.iter >= par.fulldata_max_iter );
  fulldata.previous_alldata_log_plkld = [];
  fulldata.previous_00_X0 = [];
  fulldata.previous_01_X0 = [];
  
  
  while ~( summary.finished & fulldata.finished )
    
    if par.verbose
      tic
    end
    
    % E step: estimate posteriors
    
    if ~summary.finished
      [ frame_log_post, sector_log_post, a_new.alldata_log_plkld ] = compute_post_2d2g1sr( a_new.model, summary.data );
    else
      [ frame_log_post, sector_log_post, a_new.alldata_log_plkld ] = compute_post_2d2g1sr( a_new.model, data );
    end
    
    % Verbosity
    
    if par.verbose
      
      disp( ' ' );
      
      if ~summary.finished
	disp( sprintf( [ mfilename ': model at iteration summary.iter:%d' ], summary.iter ) );
      else
	disp( sprintf( [ mfilename ': model at iteration fulldata.iter:%d' ], fulldata.iter ) );
      end
      
      disp( a_new.model );
      
      if ~summary.finished
	previous_alldata_log_plkld = summary.previous_alldata_log_plkld;
      else
	previous_alldata_log_plkld = fulldata.previous_alldata_log_plkld;
      end
      
      if isempty( previous_alldata_log_plkld )
	disp( sprintf( [ mfilename ': alldata_log_plkld:%g' ], a_new.alldata_log_plkld ) );
      else
	disp( sprintf( [ mfilename ': alldata_log_plkld:%g (variation:%g)' ], a_new.alldata_log_plkld, a_new.alldata_log_plkld - previous_alldata_log_plkld ) );
      end
      
      if ~summary.finished
	summary.previous_alldata_log_plkld = a_new.alldata_log_plkld;
      else
	fulldata.previous_alldata_log_plkld = a_new.alldata_log_plkld;
      end
      
    end
    
    
    % Look for improvement
    
    % Always update in some particular cases
    is_improving = ( ( ~summary.finished ) & isempty( summary.best ) ) | ( ( summary.finished ) & isempty( fulldata.best ) );

    if ~is_improving
    
      % General cases: compare partial likelihoods
      if ~summary.finished
	is_improving = (  summary.best.alldata_log_plkld < a_new.alldata_log_plkld - par.summary_lkld_conv );
      else
	is_improving = ( fulldata.best.alldata_log_plkld < a_new.alldata_log_plkld - par.fulldata_lkld_conv );
      end
      
    end
    
    % If improvement, then update the "best so far"
    
    if is_improving
      
      if ~summary.finished
	summary.best = a_new;
      else
	fulldata.best = a_new;
      end
      
    end
    
    
    % Termination test
    if ~summary.finished
      summary.finished  = ( ( ~is_improving ) & ( summary.iter >= par.summary_min_iter ) ) | ( summary.iter >= par.summary_max_iter );
    else
      fulldata.finished = ( ( ~is_improving ) & ( fulldata.iter >= par.fulldata_min_iter ) ) | ( fulldata.iter >= par.fulldata_max_iter );
    end
    
    % M step: update parameters

    if ~( summary.finished & fulldata.finished )

      if summary.finished & ( fulldata.iter == -1 )
	
	% Transition from the summarized data set to the full data set
	% -> pick the best model so far
	
	model = summary.best.model;
	
      else
	
	% General case: regular update
	% Update priors of silent frame and active frame
	
	tmp = my_logsum_fast( [ reshape( frame_log_post( :,:,1 ), [], 1 ) reshape( frame_log_post( :,:,2 ), [], 1 ) ] );
	tmp = exp( tmp - my_logsum_fast( tmp ) );
	a_new.model.a0 = tmp( 1 ) / sum( tmp );  % For minor numerical errors
	
	% Update priors of silent sector and active sector, given that the frame is active
	tmp = my_logsum_fast( [ reshape( sector_log_post( :,:,2 ), [], 1 ) reshape( sector_log_post( :,:,3 ), [], 1 ) ] );
	tmp = exp( tmp - my_logsum_fast( tmp ) );
	a_new.model.b0 = tmp( 1 ) / sum( tmp );  % For minor numerical errors
	
	% Update weights of the ( silent sector, silent frame ) mixture
	tmp_mat = sector_log_post( :,:,1 );
	if ~summary.finished
	  tmp      = my_logsum_fast( reshape( tmp_mat( summary.z_ind  ), [], 1 ) );
	  tmp( 2 ) = my_logsum_fast( reshape( tmp_mat( summary.nz_ind ), [], 1 ) );
	else
	  tmp      = my_logsum_fast( reshape( tmp_mat( fulldata.z_ind  ), [], 1 ) );
	  tmp( 2 ) = my_logsum_fast( reshape( tmp_mat( fulldata.nz_ind ), [], 1 ) );
	end
	tmp = exp( tmp - my_logsum_fast( tmp ) );
	a_new.model.w00d = tmp( 1 ) / sum( tmp );  % For minor numerical errors
	a_new.model.w00d = max( par.wd_floor, a_new.model.w00d ); % Flooring
	
	% Update weights of the ( silent sector, active frame ) mixture
	tmp_mat = sector_log_post( :,:,2 );
	if ~summary.finished
	  tmp      = my_logsum_fast( reshape( tmp_mat( summary.z_ind  ), [], 1 ) );
	  tmp( 2 ) = my_logsum_fast( reshape( tmp_mat( summary.nz_ind ), [], 1 ) );
	else
	  tmp      = my_logsum_fast( reshape( tmp_mat( fulldata.z_ind  ), [], 1 ) );
	  tmp( 2 ) = my_logsum_fast( reshape( tmp_mat( fulldata.nz_ind ), [], 1 ) );
	end
	tmp = exp( tmp - my_logsum_fast( tmp ) );
	a_new.model.w01d = tmp( 1 ) / sum( tmp );  % For minor numerical errors
	a_new.model.w01d = max( par.wd_floor, a_new.model.w01d ); % Flooring
	
	
	% Update the ( gamma00, beta00 ) parameters
	
	t00 = a_new.model.gamma00 * a_new.model.beta00;
	
	X0 = [ t00;
	       a_new.model.gamma00;
	       NaN;
	       NaN ];
	
	if ~summary.finished
	  
	  if ~( isempty( par.summary_inertia ) | isempty( summary.previous_00_X0 ) )
	    X0 = max( [ 1e-10; GAMMA0_FLOOR; 1e-10; 1e-10 ], X0 + par.summary_inertia * ( X0 - summary.previous_00_X0 ) );
	  end
	  summary.previous_00_X0 = X0;
	
	elseif ~fulldata.finished
	
	  if ~( isempty( par.fulldata_inertia ) | isempty( fulldata.previous_00_X0 ) )
	    X0 =  max( [ 1e-10; GAMMA0_FLOOR; 1e-10; 1e-10 ], X0 + par.fulldata_inertia * ( X0 - fulldata.previous_00_X0 ) );
	  end
	  fulldata.previous_00_X0 = X0;
	
	end
	
	negloglike_w0 = exp( sector_log_post( :,:,1 ) );
	negloglike_w1 = [];
	
	fval0 = negloglike( X0 );
	
	[ X, fval ] = fminsearch( @negloglike, X0, par.fminsearch_options );
	
	% Sanity check
	if -fval < -fval0
	  error( [ mfilename ' decreased the likelihood during ( gamma00, beta00 ) optimization!' ] );
	end
	
	a_new.model.gamma00 = X( 2 );
	a_new.model.beta00  = X( 1 ) / X( 2 );
	
	% Ensure that GammaPdf( 0, gamma00, beta00 ) = 0
	
	t00 = a_new.model.gamma00 * a_new.model.beta00;
	
	a_new.model.gamma00 = max( GAMMA0_FLOOR, a_new.model.gamma00 );
	a_new.model.beta00  = t00 / a_new.model.gamma00;
	
	
	% Update the ( gamma01, beta01, sigma11, V11 ) parameters
	
	t01 = a_new.model.gamma01 * a_new.model.beta01;
	
	X0 = [ t01;
	       a_new.model.gamma01;
	       a_new.model.sigma11;
	       a_new.model.V11 ];
	
	if ~summary.finished

	  if ~( isempty( par.summary_inertia ) | isempty( summary.previous_01_X0 ) )
	    X0 = max( [ 1e-10; GAMMA0_FLOOR; 1e-10; 1e-10 ], X0 + par.summary_inertia * ( X0 - summary.previous_01_X0 ) );
	  end
	  summary.previous_01_X0 = X0;
	  
	elseif ~fulldata.finished

	  if ~( isempty( par.fulldata_inertia ) | isempty( fulldata.previous_01_X0 ) )
	    X0 = max( [ 1e-10; GAMMA0_FLOOR; 1e-10; 1e-10 ], X0 + par.fulldata_inertia * ( X0 - fulldata.previous_01_X0 ) );
	  end
	  fulldata.previous_01_X0 = X0;
	
	end
	  
	
	negloglike_w0 = exp( sector_log_post( :,:,2 ) );
	negloglike_w1 = exp( sector_log_post( :,:,3 ) );
	
	
	fval0 = negloglike( X0 );
	
	[ X, fval ] = fminsearch( @negloglike, X0, par.fminsearch_options );
	
	if -fval < -fval0
	  error( [ mfilename ' decreased the likelihood during ( gamma01, beta01, sigma11, V11 ) optimization !' ] );
	end
	
	a_new.model.gamma01 = X( 2 );
	a_new.model.beta01  = X( 1 ) / X( 2 );

	a_new.model.sigma11 = X( 3 );
	a_new.model.V11     = X( 4 );
	
	% Ensure that GammaPdf( 0, gamma01, beta01 ) = 0
	
	t01 = a_new.model.gamma01 * a_new.model.beta01;
	
	a_new.model.gamma01 = max( GAMMA0_FLOOR, a_new.model.gamma01 );
	a_new.model.beta01  = t01 / a_new.model.gamma01;
	
      end      
    end
    
    
    % Prepare for the next iteration
    if ~summary.finished
      summary.iter = summary.iter + 1;
    elseif ~fulldata.finished
      fulldata.iter = fulldata.iter + 1;
    end

    if par.verbose
      disp( ' ' );
      disp( sprintf( [ mfilename ': iter duration was %g sec' ], toc ) );
    end
  end  % of "while" loop

  % Verbosity to conclude the training
  
  if par.verbose
    disp( ' ' );
    disp( sprintf( [ mfilename ': training finished.' ] ) )

    disp( sprintf( [ '  summary.iter:%d, summary.best.alldata_log_plkld:%g, summary.best.model:' ], summary.iter, summary.best.alldata_log_plkld ) );
    disp( summary.best.model );
    
    disp( sprintf( [ '  fulldata.iter:%d, fulldata.best.alldata_log_plkld:%g, fulldata.best.model:' ], fulldata.iter, fulldata.best.alldata_log_plkld ) );
    disp( fulldata.best.model );

  end
  

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % ( 3 ) Return values

  fulldata_model = [];
  fulldata_misc.alldata_log_plkld = [];
  if ~isempty( fulldata.best )
    fulldata_model = fulldata.best.model;
    fulldata_misc.alldata_log_plkld = fulldata.best.alldata_log_plkld;
  end
  fulldata_misc.iter = fulldata.iter;
  
  summary_model = [];
  summary_misc.alldata_log_plkld = [];
  if ~isempty( summary.best )
    summary_model = summary.best.model;
    summary_misc.alldata_log_plkld = summary.best.alldata_log_plkld;
  end
  summary_misc.iter = summary.iter

  %%% 
  
  if par.debug

    %%%%%%%%%%
    % Summarized-data model

    if ~isempty( summary.best )
      model = summary.best.model;
      
      % Show the distributions
      
      [h.w, h.x] = hist( data(:), DEBUG_HIST_NBINS );
      h.x = h.x(:);
      h.w = running_mean( h.w, 3 );
      h.w = h.w(:) / sum( h.w );
      
      [ h.frame_log_post, h.sector_log_post, h.alldata_log_plkld, h.frame_log_plkld, h.frame_log_pjoint, h.sector_log_pjoint_cond ] = compute_post_2d2g1sr( model, h.x );
      h.sector_log_pjoint = cat( 3, h.sector_log_pjoint_cond( :,:,1 ) + log( model.a0 ), h.sector_log_pjoint_cond( :,:,2 ) + log( 1 - model.a0 ), h.sector_log_pjoint_cond( :,:,3 ) + log( 1 - model.a0 ) );
      a_mat = exp( [ permute( h.sector_log_pjoint, [1 3 2] ) ] );
      a_mat = [ a_mat  sum( a_mat, 2 ) ];
      a_mat = a_mat / sum( a_mat(2:end,end) ) * sum( h.w(2:end) );
      
      a_mat( 1,1 ) = a_mat( 1,1 ) + model.a0 * model.w00d;
      a_mat( 1,2 ) = a_mat( 1,2 ) + ( 1-model.a0 ) * model.w01d;
      
      figure; set( bar( h.x, h.w ), 'edgecolor', [ 0.7 0.7 0.7 ], 'facecolor', [ 0.7 0.7 0.7 ] );
      hold on; plot( h.x, a_mat( :,1:end-1), 'linewidth', 2 );
      plot( h.x, a_mat( :,end ), 'c--', 'linewidth', 1 );
      
      xlabel( 'data' );
      title( 'summary' );
      
      % Show the distributions in log domain
      
      [hlog.w, hlog.x] = hist( log( data( find( data > 0 ) ) ), DEBUG_HIST_NBINS );
      hlog.x = hlog.x(:);
      hlog.w = running_mean( hlog.w, 3 );
      hlog.w = hlog.w(:) / sum( hlog.w );
      
      [ hlog.frame_log_post, hlog.sector_log_post, hlog.alldata_log_plkld, hlog.frame_log_plkld, hlog.frame_log_pjoint, hlog.sector_log_pjoint_cond ] = compute_post_2d2g1sr( model, exp( hlog.x ) );
      hlog.sector_log_pjoint = cat( 3, hlog.sector_log_pjoint_cond( :,:,1 ) + log( model.a0 ), hlog.sector_log_pjoint_cond( :,:,2 ) + log( 1 - model.a0 ), hlog.sector_log_pjoint_cond( :,:,3 ) + log( 1 - model.a0 ) );
      a_mat = exp( repmat( hlog.x, 1, 3 ) + [ permute( hlog.sector_log_pjoint, [1 3 2] ) ] );
      a_mat = [ a_mat  sum( a_mat, 2 ) ];
      a_mat = a_mat / sum( a_mat(2:end,end) ) * sum( hlog.w(2:end) );
      
      a_mat( 1,1 ) = a_mat( 1,1 ) + model.a0 * model.w00d;
      a_mat( 1,2 ) = a_mat( 1,2 ) + ( 1-model.a0 ) * model.w01d;
      
      figure; set( bar( hlog.x, hlog.w ), 'edgecolor', [ 0.7 0.7 0.7 ], 'facecolor', [ 0.7 0.7 0.7 ] );
      hold on; plot( hlog.x, a_mat( :,1:end-1), 'linewidth', 2 );
      plot( hlog.x, a_mat( :,end ), 'c--', 'linewidth', 1 );
      
      xlabel( 'log( data )' );
      title( 'summary' );
      
    end

    %%%%%%%%%%
    % Full-data model
    
    if ~isempty( summary.best )
      model = fulldata.best.model;
      
      % Show the distributions
      
      [h.w, h.x] = hist( data(:), DEBUG_HIST_NBINS );
      h.x = h.x(:);
      h.w = running_mean( h.w, 3 );
      h.w = h.w(:) / sum( h.w );
      
      [ h.frame_log_post, h.sector_log_post, h.alldata_log_plkld, h.frame_log_plkld, h.frame_log_pjoint, h.sector_log_pjoint_cond ] = compute_post_2d2g1sr( model, h.x );
      h.sector_log_pjoint = cat( 3, h.sector_log_pjoint_cond( :,:,1 ) + log( model.a0 ), h.sector_log_pjoint_cond( :,:,2 ) + log( 1 - model.a0 ), h.sector_log_pjoint_cond( :,:,3 ) + log( 1 - model.a0 ) );
      a_mat = exp( [ permute( h.sector_log_pjoint, [1 3 2] ) ] );
      a_mat = [ a_mat  sum( a_mat, 2 ) ];
      a_mat = a_mat / sum( a_mat(2:end,end) ) * sum( h.w(2:end) );
      
      a_mat( 1,1 ) = a_mat( 1,1 ) + model.a0 * model.w00d;
      a_mat( 1,2 ) = a_mat( 1,2 ) + ( 1-model.a0 ) * model.w01d;
      
      figure; set( bar( h.x, h.w ), 'edgecolor', [ 0.7 0.7 0.7 ], 'facecolor', [ 0.7 0.7 0.7 ] );
      hold on; plot( h.x, a_mat( :,1:end-1), 'linewidth', 2 );
      plot( h.x, a_mat( :,end ), 'c--', 'linewidth', 1 );
      
      xlabel( 'data' );
      title( 'fulldata' )
      
      % Show the distributions in log domain
      
      [hlog.w, hlog.x] = hist( log( data( find( data > 0 ) ) ), DEBUG_HIST_NBINS );
      hlog.x = hlog.x(:);
      hlog.w = running_mean( hlog.w, 3 );
      hlog.w = hlog.w(:) / sum( hlog.w );
      
      [ hlog.frame_log_post, hlog.sector_log_post, hlog.alldata_log_plkld, hlog.frame_log_plkld, hlog.frame_log_pjoint, hlog.sector_log_pjoint_cond ] = compute_post_2d2g1sr( model, exp( hlog.x ) );
      hlog.sector_log_pjoint = cat( 3, hlog.sector_log_pjoint_cond( :,:,1 ) + log( model.a0 ), hlog.sector_log_pjoint_cond( :,:,2 ) + log( 1 - model.a0 ), hlog.sector_log_pjoint_cond( :,:,3 ) + log( 1 - model.a0 ) );
      a_mat = exp( repmat( hlog.x, 1, 3 ) + [ permute( hlog.sector_log_pjoint, [1 3 2] ) ] );
      a_mat = [ a_mat  sum( a_mat, 2 ) ];
      a_mat = a_mat / sum( a_mat(2:end,end) ) * sum( hlog.w(2:end) );
      
      a_mat( 1,1 ) = a_mat( 1,1 ) + model.a0 * model.w00d;
      a_mat( 1,2 ) = a_mat( 1,2 ) + ( 1-model.a0 ) * model.w01d;
      
      figure; set( bar( hlog.x, hlog.w ), 'edgecolor', [ 0.7 0.7 0.7 ], 'facecolor', [ 0.7 0.7 0.7 ] );
      hold on; plot( hlog.x, a_mat( :,1:end-1), 'linewidth', 2 );
      plot( hlog.x, a_mat( :,end ), 'c--', 'linewidth', 1 );
      
      xlabel( 'log( data )' );
      title( 'fulldata' )
    end
    
  end
  


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function y = negloglike( X )
  
% Evaluate the negative of the tied term in log lkld of the whole
% data, that involves a Gamma + Shifted Rice mixture:
% - strictly positive data.
% - parameters gamma, beta, sigma and V.
%
% if isempty( negloglike_w1 ), Gamma is fitted alone, without shifted Rice.
  

  global data   summary  fulldata log_data_nz_ind   negloglike_w0  negloglike_w1;
  
  % Get the parameters
  
  t = X( 1 );

  gamma = X( 2 );
  beta  = t / gamma;
  
  sigma = X( 3 );
  V     = X( 4 );
  aV    = abs( V );
  
  % Impose constraint on them
  
  if ( gamma <= 1 ) | ( beta <= 0 ) | ( sigma <= 0 ) | ( V <= 0 )
    y = +Inf; return;
  end
  
  % Do it!
  
  if ~summary.finished
    grt_ind = find( summary.data > t );
  else
    grt_ind = find( data > t );
  end
  
  negloglike_w0 = negloglike_w0(:);
  negloglike_w1 = negloglike_w1(:);
  
  y = 0;
 
  
  % Gamma
  if ~isempty( negloglike_w0 )
    
    if ~summary.finished
      y = y - negloglike_w0( summary.nz_ind(:) ).' * ( ( gamma - 1 ) .* summary.log_data_nz_ind - summary.data( summary.nz_ind(:) ) / beta - gammaln( gamma ) - gamma * log( beta ) );
    else
      y = y - negloglike_w0( fulldata.nz_ind(:) ).' * ( ( gamma - 1 ) .* log_data_nz_ind - data( fulldata.nz_ind(:) ) / beta - gammaln( gamma ) - gamma * log( beta ) );
    end
  
  end
  
  
  % Shifted Rice
  if ~isempty( negloglike_w1 )
    
    if ~summary.finished
      tmp = summary.data( grt_ind ) - t;
    else
      tmp = data( grt_ind ) - t;
    end
    
    y = y - negloglike_w1( grt_ind(:) ).' * ( log( tmp(:) ) - 2 * log( sigma ) - ( tmp(:) .^ 2 + aV ^ 2 ) / ( 2 * sigma ^ 2 ) + log( besseli( 0, tmp(:) * aV / sigma ^ 2 ) ) );
    
  end
  
  % Normalize y by its number of terms,
  % so that "TolFun" may have a meaning
  
  if ~summary.finished
    y = y / numel( summary.data(:) );
  else
    y = y / numel( data(:) );
  end
  