function cc_result = s060223_cc_analysis( video_filename_list, audio_filename, audio_start_time )
  
  if nargin < 3
    error( [ mfilename ' needs 3 input arguments.' ] );
  end
  
  % Load video
  
  ncam = numel( video_filename_list );
  
  for a = 1:ncam
    
    tmp = load( video_filename_list{ a }, 'video_motion' );
    
    video_motion( a ) = tmp.video_motion;
    
    clear tmp;
    
    disp( [ mfilename ' loaded "' video_filename_list{ a } '"' ] );
    
  end
  
  % Load audio
  
  load( audio_filename, 'ssm_result' );

  disp( [ mfilename ' loaded "' audio_filename '"' ] );
  
  if ~isfield( ssm_result, 'model_2d2g1sr' )

    disp( [ mfilename ': you forgot to fit a model on audio data. I''ll do it for you.' ] );
    
    s060223_fit_audio_model( audio_filename );
    load( audio_filename, 'ssm_result' );
    
    if ~isfield( ssm_result, 'model_2d2g1sr' )
      error( [ mfilename ' failed fitting a model on audio data.' ] );
    end
    
  end
  
  % Compute posteriors for audio
  
  [ tmp, audio_log_post_0 ] = compute_post_2d2g1sr( ssm_result.model_2d2g1sr, ssm_result.activeness );
  
  audio_post_0 = exp( audio_log_post_0( :,:,3 ) );  

  clear audio_log_post_0;
  
  % At this point we have two small issues:
  % - audio and video start times differ.
  % - audio and video frequencies differ.
  %
  % -> merge audio frames
  
  audio_framelength_sec = ssm_result.framelength / ssm_result.fs;
  audio_frameshift_sec  = ssm_result.frameshift  / ssm_result.fs;
  
  video_frameshift_sec  = [];
  
  for a = 1:ncam
    video_frameshift_sec( end+1 ) = video_motion( a ).par.interval_sec;
  end
  
  video_frameshift_sec = unique( video_frameshift_sec );
  
  if numel( video_frameshift_sec ) ~= 1
    error( [ mfilename ' needs all video frame rates being equal.' ] );
  end
  
  if audio_frameshift_sec > video_frameshift_sec
    error( [ mfilename ' needs audio_frameshift_sec < video_frameshift_sec.' ] );
  end
  
  % Before merging
  
  t0 = audio_start_time + audio_framelength_sec / 2 + audio_frameshift_sec * ( 0:size( audio_post_0, 2 ) - 1 );
  
  % After merging
  
  audio_t = t0(1):video_frameshift_sec:t0(end);
  
  % Ok, now we can do the merge
  
  nsectors = size( audio_post_0, 1 );
  
  nframes_0 = size( audio_post_0, 2 );
  
  audio_post = zeros( nsectors, numel( audio_t ) );
  
  n = ceil( video_frameshift_sec / audio_frameshift_sec );
  
  for a = 1:numel( audio_t )
    
    % Find the frames for this time
    
    [ tmp, ind_max ] = min( abs( t0 - audio_t( a ) ) );
    
    ind_max = min( nframes_0, ind_max );
    
    ind_min = max( 1, ind_max - n + 1 );
    
    % Average the posteriors
    
    audio_post( :, a ) = mean( audio_post_0( :, ind_min:ind_max ), 2 );
    
  end
  
  % Now we can do cross-correlation analysis
  
  
  cc_result = [];
  cc_result.video_filename_list = video_filename_list;
  cc_result.audio_filename      = audio_filename;
  cc_result.audio_start_time    = audio_start_time;
  
  % Between each camera and the audio
  
  cc_result.list = [];
  
  for a = 1:ncam
    
    ind = numel( cc_result.list ) + 1;
    
    cc_result.list( ind ).description = sprintf( 'camera #%d and audio', a );

    disp( ' ' );
    disp( [ mfilename ' is looking at correlation between ' cc_result.list( ind ).description '...' ] );
    
    t1    = video_motion( a ).timestamp_sec;
    data1 = video_motion( a ).mat;
    
    t2    = audio_t;
    data2 = audio_post;
    
    cc_result.list( ind ).size1 = size( data1 );
    cc_result.list( ind ).size2 = size( data2 );
    cc_result.list( ind ).mat   = cc_analysis( t1, data1, t2, data2, video_frameshift_sec );
    
  end
  
  % Between each pair of cameras
  
  for a = 1:ncam
    
    for b = a+1:ncam
      
      ind = numel( cc_result.list ) + 1;
      
      cc_result.list( ind ).description = sprintf( 'camera #%d and camera #%d', a, b );
      
      disp( ' ' );
      disp( [ mfilename ' is looking at correlation between ' cc_result.list( ind ).description '...' ] );
      
      t1    = video_motion( a ).timestamp_sec;
      data1 = video_motion( a ).mat;
      
      t2    = video_motion( b ).timestamp_sec;
      data2 = video_motion( b ).mat;
      
      cc_result.list( ind ).size1 = size( data1 );
      cc_result.list( ind ).size2 = size( data2 );
      cc_result.list( ind ).mat   = cc_analysis( t1, data1, t2, data2, video_frameshift_sec );
      
    end
    
  end
  
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function mat = cc_analysis( t1, data1, t2, data2, interval_sec )

  if nargin < 5
    error( [ mfilename '.cc_analysis() needs 5 input arguments.' ] );
  end
  
  size1 = size( data1 );
  size2 = size( data2 );
  
  % Where we will store the correlation coefficient
  % between each dimension of modality 1
  %     and each dimension of modality 2.

  mat_size = [ size1( 1:end-1 ) size2( 1:end-1 ) ];
  
  mat = repmat( NaN, mat_size );
  
  % First, we match the timestamps
  
  nframes1 = size1( end );
  
  if nframes1 ~= numel( t1 )
    error( [ mfilename '.cc_analysis(): inconsistency between t1 and data1.' ] );
  end
  
  nframes2 = size2( end );
  
  if nframes2 ~= numel( t2 )
    error( [ mfilename '.cc_analysis(): inconsistency between t2 and data2.' ] );
  end
  
  match_ind = repmat( NaN, 1, nframes2 );
  
  for a = 1:nframes2
    
    [ tmp, ind ] = min( abs( t1 - t2( a ) ) );
    
    % 1-frame error tolerance
      
    if tmp < interval_sec
      
      match_ind( a ) = ind;
      
    end
    
  end
  
  % Select the corresponding frames
  
  ind = find( ~isnan( match_ind ) );
  
  frame_ind1 = match_ind( ind );
  frame_ind2 = ind;
  
  % Second, we flatten the problem
  % -> each modality becomes temporarily 1-D
  
  dim1 = prod( size1( 1:end-1 ) );
  dim2 = prod( size2( 1:end-1 ) );
  
  mat   = reshape( mat, [ dim1 dim2 ] );
  
  data1 = reshape( data1, dim1, nframes1 );
  data2 = reshape( data2, dim2, nframes2 );

  % Correlation between each pair of dimensions
  
  chrono = chrono_start( dim1 * dim2, 30 );
  
  iter = 0;
  
  for a1 = 1:dim1
    
    v1 = data1( a1, frame_ind1 );
    
    for a2 = 1:dim2
      
      v2 = data2( a2, frame_ind2 );
      
      tmp = corrcoef( v1(:), v2(:) );
      
      % Store it
      
      mat( a1, a2 ) = tmp( 1, 2 );
      
      %
      
      iter = iter + 1;
      
      chrono = chrono_check( chrono, iter );
      
    end
    
  end
  
  chrono_stop( chrono );
  
  % Reshape the correlation matrix back to initial dimensions
  
  mat = reshape( mat, mat_size );
  